# function to create graphs frmo the GRain output
# R code written by Gabor Meszaros, gabor.meszaros@boku.ac.at
# 22/01/2015

# To reset graphical parameters of R use dev.off() in your script
# Warning: This deletes all non-saved plots! 

##########################################################
# function for heatmaps 
partInbreedingMap=function(partInbrFileName,heatMapTitle){
  
  #install and load packages if required
  if (!require("gplots")) {
    install.packages("gplots", dependencies = TRUE)
    library(gplots)
  }
  if (!require("RColorBrewer")) {
    install.packages("RColorBrewer", dependencies = TRUE)
    library(RColorBrewer)
  }
  
  #read in the file with partial inbreeding coefficients
  firstLine=as.character(readLines(partInbrFileName, n=1))
  inNames=strsplit(firstLine,split=" ")
  inNames=unlist(inNames)
  inNames=c("animal","sire","dam","total",inNames[inNames!=""])
  
  inFile=read.table(partInbrFileName,skip=1,header=F) 
  colnames(inFile)=inNames
  rownames(inFile)=as.character(inFile$animal)
  
  # extract partial inbreeding coefficients to be plotted
  myMat=as.matrix(inFile[,4:ncol(inFile)],ncol=ncol(inFile)-4)
  # exclude funder animals appearing in the X axis, partial inbreeding 0 by definition 
  myMat=myMat[row.names(myMat)[!row.names(myMat) %in% inNames[5:length(inNames)]],]
   
  # creates a own color palette from red to green
  #my_palette <- colorRampPalette(c("lightyellow", "yellow", "red"))(n = 299)
  # grayscale
  my_palette <- colorRampPalette(gray.colors(3, start = 0.9, end = 0.3, gamma = 2.2))(n = 299)
  
  # create heatmap to visualize all inbreeding coefficients
  heatmap.2(myMat,               # input data 
#           cellnote = myMat,    # same data set for cell labels
            main = heatMapTitle, # heat map title
            notecol="black",     # change font color of cell labels to black
            density.info="none", # turns off density plot inside color legend
            trace="none",        # turns off trace lines inside the heat map
            #margins =c(12,9),   # widens margins around plot
            col=my_palette,      # use on color palette defined earlier 
            # breaks=col_breaks, # enable color transition at specified limits
            dendrogram="none",   # no dendrogram
            Rowv="NA",           # turn off row clustering- time consuming
            Colv="NA")           # turn off column clustering
}


##########################################################
# main function
# execute GRain and plot results
GRain=function(execName="GRain.exe",pedFile="",option="",nAncestors="",ancestorFile="no",nDrops="",yearCol=0,UNIXlike=F){
  
  #change character input to GRain specific input numbers
  if (option=="totalOnly") {inOption="1"}
  if (option=="fAndPartial") {inOption="2"}
  if (option=="faBallouAndPartial") {inOption="3"}
  if (option=="AHCAndPartial") {inOption="4"}
  if (option=="faKalinowskiAndPartial") {inOption="5"}
  if (option=="all") {inOption="0"}
  
  #if file with ancestors specified, read in and check size
  if (ancestorFile!="no") {
    ancestors=read.table(ancestorFile,header=F)
    # set number of ancestors to ncol(ancestors) to show all important data to the user
    # in case lower number of ancestors were specified
    if (nAncestors<nrow(ancestors)) {
      nAncestors=nrow(ancestors)
      print(paste("Warning: Number of ancestors changed to",nAncestors,"!",sep=" "))
      print(paste("Partial inbreeding coefficients computed for all individuals from",ancestorFile,sep=" "))
    }
  }
  
  
  # run GRain
  # adding "./" to executable to run it in UNIX like systems
  if (UNIXlike) execName=paste("./",execName,sep="")
  # execute GRain on command line
  # differenc nubmers of input arguments are needed when only the total F is computed
  if (inOption=="1"){
    system(execName,input=c(pedFile,inOption,nDrops))
  } else {
    system(execName,input=c(pedFile,inOption,nAncestors,ancestorFile,nDrops))
  }
  
  
  # visualize GRain results
#  #######
#  # Plot: Total inbeeding coefficients from all methods
  if (inOption=="1" | inOption=="0") {
    #read in GRain result file
    ped=read.table(paste("tot_",pedFile,sep=""),header=T,colClasses="numeric")
#    #plot results    
#    plot(ped$animal,ped$f.classical,type="p",pch=15,col="green",ylim=range(ped[,4:7]),
#         xlab="Individual",ylab="Inbreeding coefficient",main="Total inbreeding coefficients")
#    points(ped$animal,ped$fa.Ballou,type="p",pch=16,col="blue")
#    points(ped$animal,ped$fa.history,type="p",pch=20,col="grey")
#    points(ped$animal,ped$fa.Kalinowski,type="p",pch=4,col="gold")
#    legend("topleft", legend=c("f classical","fa Ballou","AHC","fa Kalinowski"), 
#           pch=c(15,16,20,4), col=c("green","blue","grey","gold"))
#
#    #######
    #Plot: Trends according to birth years (if birth year columns specified)
    if (yearCol>0) {
      #read in original pedigree file
      orgPed=read.table(pedFile)
      # extract birth year and merge with results
      extYear=orgPed[,c(1,yearCol)]
      colnames(extYear)=c("animal","birthYear")
      allRes=merge(extYear,ped,by="animal")
      
      # axis boundaries
      minY=min(allRes[,c(5,6)]) #calssical and Ballou F considered
      maxY=max(allRes[,c(5,6)]) #calssical and Ballou F considered
      
      #plot inbreeding trends by year
      #plot inbreeding trends by year
      par(mfrow=c(2,2))
      boxplot(f.classical~birthYear,data=allRes,
              xlab="Year",ylab="Inbreeding coefficient",ylim=c(minY,maxY))
      boxplot(fa.Ballou~birthYear,data=allRes,
              xlab="Year",ylab="Ancestral inbreeding coefficient - Ballou",ylim=c(minY,maxY))
      boxplot(AHC~birthYear,data=allRes,xlab="Year",ylab="Ancestral history coefficient")
      boxplot(fa.Kalinowski~birthYear,data=allRes,xlab="Year",ylab="Ancestral inbreeding coefficient - Kalinowski",ylim=c(minY,maxY))
      par(mfrow=c(1,1))
	}
  } 
  
#  #######
#  # Heatmaps for total and partial inbreeding coefficients
#  #  Time consuming operation with large data sets 
#  #######
#  # Plot: Total and partial inbreeding - classical
#  if (inOption=="2" | inOption=="0") {
#    partInbreedingMap(partInbrFileName=paste("F_",pedFile,sep=""),heatMapTitle="Inbreeding - Classical")
#  }
#  #######  
#  # Plot: Total and partial inbreeding - Ballou
#  if (inOption=="3" | inOption=="0") {
#    partInbreedingMap(partInbrFileName=paste("Fa_bal_",pedFile,sep=""),heatMapTitle="Inbreeding - Ballou")
#  }
#  #######
#  # Plot: Total and partial inbreeding - History - AHC
#  if (inOption=="4" | inOption=="0") {
#    partInbreedingMap(partInbrFileName=paste("AHC_",pedFile,sep=""),heatMapTitle="AHC")
#  }
#  #######
#  # Plot: Total and partial inbreeding - Kalinowski
#  if (inOption=="5" | inOption=="0") {
#    partInbreedingMap(partInbrFileName=paste("Fa_kal_",pedFile,sep=""),heatMapTitle="Inbreeding - Kalinowski")
#  }
  
  
}

